@@ -36,5 +36,5 @@ PROGRAM alg
     this_step = istp
     CALL invoke_0_inc_field(fld1, nx, ny, this_step)
   END DO
-  WRITE(*, FMT = *) "nsteps = ", nsteps, "field(2,2) = ", fld1 % data(2, 2)
+  WRITE(*, *) "nsteps = ", nsteps, "field(2,2) = ", fld1 % data(2, 2)
 END PROGRAM alg
\ No newline at end of file
