@@ -4,7 +4,7 @@ module inc_field_0_mod
   use argument_mod
   use grid_mod, only : go_offset_sw
   implicit none
-  TYPE, EXTENDS(kernel_type) :: inc_field
+  type, extends(kernel_type), public :: inc_field
   TYPE(go_arg), DIMENSION(4) :: meta_args = (/go_arg(go_write, go_ct, go_pointwise), go_arg(go_read, go_i_scalar, go_pointwise), &
 &go_arg(go_read, go_i_scalar, go_pointwise), go_arg(go_read, go_i_scalar, go_pointwise)/)
   INTEGER :: iterates_over = go_internal_pts
@@ -12,6 +12,8 @@ module inc_field_0_mod
   CONTAINS
   PROCEDURE, NOPASS :: code => inc_field_0_code
 END TYPE inc_field
+  public
+
   public :: inc_field_0_code
 
   contains
