@@ -36,10 +36,9 @@
 ''' Module providing a transformation script that converts the Schedule of
     the first Invoke to use OpenCL. '''
 
-from psyclone.transformations import OCLTrans
 from psyclone.psyir.transformations import \
     FoldConditionalReturnExpressionsTrans
-from psyclone.domain.gocean.transformations import \
+from psyclone.domain.gocean.transformations import GOOpenCLTrans, \
     GOMoveIterationBoundariesInsideKernelTrans
 
 
@@ -54,7 +53,7 @@ def trans(psy):
     :rtype: :py:class:`psyclone.psyGen.PSy`
 
     '''
-    ocl_trans = OCLTrans()
+    ocl_trans = GOOpenCLTrans()
     fold_trans = FoldConditionalReturnExpressionsTrans()
     move_boundaries_trans = GOMoveIterationBoundariesInsideKernelTrans()
 
@@ -62,11 +61,8 @@ def trans(psy):
     invoke = psy.invokes.invoke_list[0]
     sched = invoke.schedule
 
-    # Transform the Schedule
-    ocl_trans.apply(sched, options={"end_barrier": True})
-
     # Provide kernel-specific OpenCL optimization options
-    for kern in sched.kernels():
+    for idx, kern in enumerate(sched.kernels()):
         # Move the PSy-layer loop boundaries inside the kernel as a kernel
         # mask, this allows to iterate through the whole domain
         move_boundaries_trans.apply(kern)
@@ -74,6 +70,12 @@ def trans(psy):
         # previous transformation
         fold_trans.apply(kern.get_kernel_schedule())
         # Specify the OpenCL queue and workgroup size of the kernel
-        kern.set_opencl_options({"queue_number": 1, 'local_size': 4})
+        # In this case we dispatch each kernel in a different queue to check
+        # that the output code has the necessary barriers to guarantee the
+        # kernel execution order.
+        kern.set_opencl_options({"queue_number": idx+1, 'local_size': 4})
+
+    # Transform the Schedule
+    ocl_trans.apply(sched, options={"end_barrier": True})
 
     return psy
