@@ -34,6 +34,7 @@
 # Author: R. W. Ford, STFC Daresbury Laboratory
 # Modified: I. Kavcic, Met Office
 # Modified by J. Henrichs, Bureau of Meteorology
+# Modified by S. Siso, STFC Daresbury Laboratory
 
 '''File containing a PSyclone transformation script for the Dynamo0.3
 API to apply loop fusion generically. Fusion is attempted for all
@@ -67,8 +68,7 @@ def trans(psy):
             node = schedule.children[idx]
             prev_node = schedule.children[idx-1]
             try:
-                schedule, _ = lf_trans.apply(prev_node, node,
-                                             {"same_space": True})
+                lf_trans.apply(prev_node, node, {"same_space": True})
                 local_fused += 1
             except TransformationError:
                 pass
@@ -77,7 +77,6 @@ def trans(psy):
         if local_fused > 0:
             print("After fusing ...")
             schedule.view()
-            invoke.schedule = schedule
 
     print("Fused {0} loops".format(total_fused))
     return psy
