@@ -42,8 +42,8 @@ kernel calls.
 from __future__ import print_function, absolute_import
 import abc
 
-from psyclone.core.access_type import AccessType
-from psyclone.domain.lfric import LFRicArgDescriptor
+from psyclone.core import AccessType, Signature
+from psyclone.domain.lfric import LFRicConstants
 from psyclone.errors import GenerationError, InternalError
 
 
@@ -92,10 +92,10 @@ class ArgOrdering(object):
         self._arglist.append(var_name)
         if var_accesses is not None:
             if var_access_name:
-                var_accesses.add_access(var_access_name, mode,
+                var_accesses.add_access(Signature(var_access_name), mode,
                                         self._kern)
             else:
-                var_accesses.add_access(var_name, mode,
+                var_accesses.add_access(Signature(var_name), mode,
                                         self._kern)
 
     def extend(self, list_var_name, var_accesses=None,
@@ -119,7 +119,7 @@ class ArgOrdering(object):
         self._arglist.extend(list_var_name)
         if var_accesses:
             for var_name in list_var_name:
-                var_accesses.add_access(var_name, mode, self._kern)
+                var_accesses.add_access(Signature(var_name), mode, self._kern)
 
     @property
     def num_args(self):
@@ -184,12 +184,15 @@ class ArgOrdering(object):
         # this quantity for *every* operator it encounters.
         # if self._kern.arguments.has_operator(op_type="gh_operator"):
         #     self.mesh_ncell3d()
+
         # Pass the number of columns in the mesh if this kernel operates on
-        # the 'domain' or has a CMA operator argument
-        if (self._kern.iterates_over == "domain" or
-                self._kern.arguments.has_operator(
-                    op_type="gh_columnwise_operator")):
-            self.mesh_ncell2d(var_accesses=var_accesses)
+        # the 'domain' or has a CMA operator argument. For the former we
+        # exclude halo columns.
+        if self._kern.iterates_over == "domain":
+            self._mesh_ncell2d_no_halos(var_accesses=var_accesses)
+
+        if self._kern.arguments.has_operator(op_type="gh_columnwise_operator"):
+            self._mesh_ncell2d(var_accesses=var_accesses)
 
         if self._kern.is_intergrid:
             # Inter-grid kernels require special arguments
@@ -203,7 +206,7 @@ class ArgOrdering(object):
         # scalar). If the argument is a field or field vector and also
         # has a stencil access then also call appropriate stencil
         # methods.
-
+        const = LFRicConstants()
         for arg in self._kern.arguments.args:
             if arg.is_field:
                 if arg.vector_size > 1:
@@ -250,7 +253,7 @@ class ArgOrdering(object):
                 raise GenerationError(
                     "ArgOrdering.generate(): Unexpected argument type found. "
                     "Expected one of '{0}' but found '{1}'".
-                    format(LFRicArgDescriptor.VALID_ARG_TYPE_NAMES,
+                    format(const.VALID_ARG_TYPE_NAMES,
                            arg.argument_type))
         # For each function space (in the order they appear in the
         # metadata arguments)
@@ -369,9 +372,21 @@ class ArgOrdering(object):
         '''
 
     @abc.abstractmethod
-    def mesh_ncell2d(self, var_accesses=None):
-        '''Add the number of columns in the mesh to the argument list and if
-        supplied stores this access in var_accesses.
+    def _mesh_ncell2d(self, var_accesses=None):
+        '''Add the number of columns in the mesh (including halos) to the
+        argument list and stores this access in var_accesses (if supplied).
+
+        :param var_accesses: optional VariablesAccessInfo instance to store \
+            the information about variable accesses.
+        :type var_accesses: \
+            :py:class:`psyclone.core.access_info.VariablesAccessInfo`
+
+        '''
+
+    @abc.abstractmethod
+    def _mesh_ncell2d_no_halos(self, var_accesses=None):
+        '''Add the number of columns in the mesh (excluding halos) to the
+        argument list and stores this access in var_accesses (if supplied).
 
         :param var_accesses: optional VariablesAccessInfo instance to store \
             the information about variable accesses.
@@ -547,10 +562,11 @@ class ArgOrdering(object):
         :raises InternalError: if the argument is not a recognised scalar type.
 
         '''
+        const = LFRicConstants()
         if not scalar_arg.is_scalar:
             raise InternalError(
                 "Expected argument type to be one of {0} but got '{1}'".
-                format(LFRicArgDescriptor.VALID_SCALAR_NAMES,
+                format(const.VALID_SCALAR_NAMES,
                        scalar_arg.argument_type))
 
         self.append(scalar_arg.name, var_accesses, mode=scalar_arg.access)
@@ -735,6 +751,8 @@ class ArgOrdering(object):
 
         '''
         if self._kern.reference_element.properties:
+            # Avoid circular import
+            # pylint: disable=import-outside-toplevel
             from psyclone.dynamo0p3 import DynReferenceElement
             refelem_args = DynReferenceElement(self._kern).kern_args()
             self.extend(refelem_args, var_accesses)
