@@ -33,6 +33,7 @@
 # -----------------------------------------------------------------------------
 # Author A. R. Porter, STFC Daresbury Lab
 # Modified I. Kavcic, Met Office
+# Modified by J. Henrichs, Bureau of Meteorology
 
 ''' This module implements the support for 'built-in' operations in the
     PSyclone LFRic (Dynamo 0.3) API. Each supported built-in is implemented
@@ -41,25 +42,22 @@
     a given built-in call. '''
 
 from __future__ import absolute_import
+import abc
+import six
 from psyclone.core.access_type import AccessType
 from psyclone.psyGen import BuiltIn
+from psyclone.psyir.symbols import DataSymbol, INTEGER_SINGLE_TYPE
+from psyclone.psyir.nodes import Assignment, Reference, StructureReference, \
+    BinaryOperation
 from psyclone.parse.utils import ParseError
-from psyclone.domain.lfric import LFRicArgDescriptor
-from psyclone.f2pygen import AssignGen
+from psyclone.domain.lfric import LFRicConstants
+from psyclone.f2pygen import AssignGen, PSyIRGen
 from psyclone.configuration import Config
 
 # The name of the file containing the meta-data describing the
 # built-in operations for this API
 BUILTIN_DEFINITIONS_FILE = "lfric_builtins_mod.f90"
 
-# The types of argument that are valid for built-in kernels in the
-# LFRic API
-VALID_BUILTIN_ARG_TYPES = LFRicArgDescriptor.VALID_FIELD_NAMES + \
-    LFRicArgDescriptor.VALID_SCALAR_NAMES
-
-# Valid LFRic iteration spaces for built-in kernels
-BUILTIN_ITERATION_SPACES = ["dof"]
-
 
 # Function to return the built-in operations that we support for this API.
 # The meta-data describing these kernels is in lfric_builtins_mod.f90.
@@ -123,9 +121,12 @@ class LFRicBuiltInCallFactory(object):
         builtin = BUILTIN_MAP[call.func_name]()
 
         # Create the loop over DoFs
+        # Avoid circular import
+        # pylint: disable=import-outside-toplevel
         from psyclone.dynamo0p3 import DynLoop
+        const = LFRicConstants()
         dofloop = DynLoop(parent=parent,
-                          loop_type=BUILTIN_ITERATION_SPACES[0])
+                          loop_type=const.BUILTIN_ITERATION_SPACES[0])
 
         # Use the call object (created by the parser) to set-up the state
         # of the infrastructure kernel
@@ -142,10 +143,11 @@ class LFRicBuiltInCallFactory(object):
         return dofloop
 
 
-# TODO #1140 'LFRicBuiltIn' should be an abstract class.
+@six.add_metaclass(abc.ABCMeta)
 class LFRicBuiltIn(BuiltIn):
     '''
-    Parent class for a call to an LFRic Built-in.
+    Abstract base class for a node representing a call to an LFRic Built-in.
+
     '''
     def __init__(self):
         # Builtins do not accept quadrature
@@ -154,29 +156,30 @@ class LFRicBuiltIn(BuiltIn):
         self.mesh = None
         super(LFRicBuiltIn, self).__init__()
 
+    @abc.abstractmethod
     def __str__(self):
-        ''' :raises NotImplementedError: if the method is called. '''
-        raise NotImplementedError("LFRicBuiltIn.__str__ must be overridden")
+        ''' Must be overridden by sub class. '''
 
     def load(self, call, parent=None):
         '''
         Populate the state of this object using the supplied call object.
 
-        :param call: The BuiltIn object from which to extract information
+        :param call: The BuiltIn object from which to extract information \
                      about this built-in call.
         :type call: :py:class:`psyclone.parse.algorithm.BuiltInCall`
-        :param parent: The parent node of the kernel call in the AST
+        :param parent: The parent node of the kernel call in the PSyIR \
                        we are constructing. This will be a loop.
         :type parent: :py:class:`psyclone.dynamo0p3.DynLoop`
 
         '''
+        # Avoid circular import
+        # pylint: disable=import-outside-toplevel
         from psyclone.dynamo0p3 import FSDescriptors, DynKernelArguments
         BuiltIn.load(self, call, DynKernelArguments, parent)
         self.arg_descriptors = call.ktype.arg_descriptors
         self._func_descriptors = call.ktype.func_descriptors
         self._fs_descriptors = FSDescriptors(call.ktype.func_descriptors)
-        self._idx_name = \
-            self.root.symbol_table.symbol_from_tag("dof_loop_idx", "df").name
+        self._idx_name = self.get_dof_loop_index_symbol().name
         # Check that this built-in kernel is valid
         self._validate()
 
@@ -187,6 +190,8 @@ class LFRicBuiltIn(BuiltIn):
         :raises ParseError: if a built-in call does not iterate over DoFs.
         :raises ParseError: if an argument to a built-in kernel is not \
                             one of valid argument types.
+        :raises ParseError: if an argument to a built-in kernel has \
+                            an invalid data type.
         :raises ParseError: if a built-in kernel writes to more than \
                             one argument.
         :raises ParseError: if a built-in kernel does not have at least \
@@ -196,8 +201,9 @@ class LFRicBuiltIn(BuiltIn):
                             built-in do not have the same data type.
 
         '''
+        const = LFRicConstants()
         # Check that our assumption that we're looping over DoFs is valid
-        if self.iterates_over not in BUILTIN_ITERATION_SPACES:
+        if self.iterates_over not in const.BUILTIN_ITERATION_SPACES:
             raise ParseError(
                 "In the LFRic API built-in calls must operate on "
                 "DoFs but found '{0}' for {1}.".
@@ -210,18 +216,26 @@ class LFRicBuiltIn(BuiltIn):
         data_types = set()
         for arg in self.arg_descriptors:
             # Check valid argument types
-            if arg.argument_type not in VALID_BUILTIN_ARG_TYPES:
+            if arg.argument_type not in const.VALID_BUILTIN_ARG_TYPES:
                 raise ParseError(
                     "In the LFRic API an argument to a built-in kernel "
                     "must be one of {0} but kernel '{1}' has an argument of "
-                    "type '{2}'.".format(VALID_BUILTIN_ARG_TYPES, self.name,
-                                         arg.argument_type))
+                    "type '{2}'.".format(const.VALID_BUILTIN_ARG_TYPES,
+                                         self.name, arg.argument_type))
+            # Check valid data types
+            if arg.data_type not in const.VALID_BUILTIN_DATA_TYPES:
+                raise ParseError(
+                    "In the LFRic API an argument to a built-in kernel "
+                    "must have one of {0} as a data type but kernel '{1}' "
+                    "has an argument of data type '{2}'.".
+                    format(const.VALID_BUILTIN_DATA_TYPES,
+                           self.name, arg.data_type))
             # Built-ins update fields DoF by DoF and therefore can have
             # WRITE/READWRITE access
             if arg.access in [AccessType.WRITE, AccessType.SUM,
                               AccessType.READWRITE]:
                 write_count += 1
-            if arg.argument_type in LFRicArgDescriptor.VALID_FIELD_NAMES:
+            if arg.argument_type in const.VALID_FIELD_NAMES:
                 field_count += 1
                 spaces.add(arg.function_space)
                 data_types.add(arg.data_type)
@@ -297,8 +311,21 @@ class LFRicBuiltIn(BuiltIn):
         return None
 
     def gen_code(self, parent):
-        raise NotImplementedError("LFRicBuiltIn.gen_code must be overridden")
+        '''
+        Generates LFRic API specific PSy code (in the form of f2pygen AST
+        nodes) for a call to this Built-in. This method must be overridden
+        if a sub-class does not yet implement lower_to_language_level().
 
+        :param parent: Node in f2pygen tree to which to add call.
+        :type parent: :py:class:`psyclone.f2pygen.BaseGen`
+
+        '''
+        # TODO #1010 Ultimately this routine will be removed once the LFRic
+        # PSyIR has been fully migrated to use the PSyIR backends.
+        # For now we create an f2pygen node from the PSyIR of this routine.
+        parent.add(PSyIRGen(parent, self))
+
+    @property
     def cma_operation(self):
         '''
         Built-ins do not perform operations with Column-Matrix-Assembly
@@ -308,7 +335,6 @@ class LFRicBuiltIn(BuiltIn):
         :rtype: NoneType
 
         '''
-
         return None
 
     @property
@@ -332,6 +358,54 @@ class LFRicBuiltIn(BuiltIn):
         '''
         return self._fs_descriptors
 
+    def get_dof_loop_index_symbol(self):
+        '''
+        Finds or creates the symbol representing the index in any loops
+        over dofs.
+
+        :returns: symbol representing the dof loop index.
+        :rtype: :py:class:`psyclone.psyir.symbols.DataSymbol`
+
+        '''
+        table = self.scope.symbol_table
+        # The symbol representing the loop index is created in the DynLoop
+        # constructor.
+        # TODO #696 - 'df' should have KIND i_def.
+        return table.symbol_from_tag(tag="dof_loop_idx", root_name="df",
+                                     symbol_type=DataSymbol,
+                                     datatype=INTEGER_SINGLE_TYPE)
+
+    def get_indexed_field_argument_references(self):
+        '''
+        Creates a dof-indexed StructureReference for each of the field
+        arguments to this Builtin kernel. e.g. if the kernel has a field
+        argument named 'fld1' then this routine will create a
+        StructureReference for 'fld1%data(df)' where 'df' is the dof-loop
+        variable.
+
+        :returns: a reference to the 'df'th element of each kernel argument \
+                  that is a field.
+        :rtype: list of :py:class:`psyclone.psyir.nodes.StructureReference`
+
+        '''
+        idx_sym = self.get_dof_loop_index_symbol()
+
+        return [StructureReference.create(
+            arg.psyir_expression().symbol, [("data", [Reference(idx_sym)])])
+                for arg in self._arguments.args if arg.is_field]
+
+    def get_scalar_argument_references(self):
+        '''
+        Finds or creates either a Reference (for a symbol) or PSyIR (for a
+        literal expression) for any scalar arguments to this Builtin kernel.
+
+        :returns: a Reference or PSyIR expression for each scalar kernel \
+            argument.
+        :rtype: list of subclasses of `:py:class:`psyclone.psyir.nodes.Node`
+
+        '''
+        return [arg.psyir_expression() for arg in self._arguments.args
+                if arg.is_scalar]
 
 # ******************************************************************* #
 # ************** Built-ins for real-valued fields ******************* #
@@ -350,22 +424,22 @@ class LFRicXPlusYKern(LFRicBuiltIn):
     def __str__(self):
         return "Built-in: Add real-valued fields"
 
-    def gen_code(self, parent):
+    def lower_to_language_level(self):
         '''
-        Generates LFRic API specific PSy code for a call to the
-        X_plus_Y Built-in.
-
-        :param parent: Node in f2pygen tree to which to add call.
-        :type parent: :py:class:`psyclone.f2pygen.BaseGen`
+        Lowers this LFRic-specific built-in kernel to language-level PSyIR.
+        This BuiltIn node is replaced by an Assignment node.
 
         '''
-        # We add each element of f2 to the corresponding element of f1
-        # and store the result in f3 (real-valued fields).
-        field_name3 = self.array_ref(self._arguments.args[0].proxy_name)
-        field_name1 = self.array_ref(self._arguments.args[1].proxy_name)
-        field_name2 = self.array_ref(self._arguments.args[2].proxy_name)
-        parent.add(AssignGen(parent, lhs=field_name3,
-                             rhs=field_name1 + " + " + field_name2))
+        # Get indexed references for each of the field (proxy) arguments.
+        arg_refs = self.get_indexed_field_argument_references()
+
+        # Create the PSyIR for the kernel:
+        #      proxy0%data(df) = proxy1%data(df) + proxy2%data(df)
+        rhs = BinaryOperation.create(BinaryOperation.Operator.ADD,
+                                     arg_refs[1], arg_refs[2])
+        assign = Assignment.create(arg_refs[0], rhs)
+        # Finally, replace this kernel node with the Assignment
+        self.replace_with(assign)
 
 
 class LFRicIncXPlusYKern(LFRicBuiltIn):
@@ -375,21 +449,23 @@ class LFRicIncXPlusYKern(LFRicBuiltIn):
     def __str__(self):
         return "Built-in: Increment a real-valued field"
 
-    def gen_code(self, parent):
+    def lower_to_language_level(self):
         '''
-        Generates LFRic API specific PSy code for a call to the
-        inc_X_plus_Y Built-in.
-
-        :param parent: Node in f2pygen tree to which to add call.
-        :type parent: :py:class:`psyclone.f2pygen.BaseGen`
+        Lowers this LFRic-specific built-in kernel to language-level PSyIR.
+        This BuiltIn node is replaced by an Assignment node.
 
         '''
-        # We add each element of f1 to the corresponding element of f2
-        # and store the result back in f1 (real-valued fields).
-        field_name1 = self.array_ref(self._arguments.args[0].proxy_name)
-        field_name2 = self.array_ref(self._arguments.args[1].proxy_name)
-        parent.add(AssignGen(parent, lhs=field_name1,
-                             rhs=field_name1 + " + " + field_name2))
+        # Get indexed refs for both of the field (proxy) arguments.
+        arg_refs = self.get_indexed_field_argument_references()
+
+        # Create the PSyIR for the kernel:
+        #      proxy0%data(df) = proxy0%data(df) + proxy1%data(df)
+        lhs = arg_refs[0]
+        rhs = BinaryOperation.create(BinaryOperation.Operator.ADD, lhs.copy(),
+                                     arg_refs[1])
+        assign = Assignment.create(lhs, rhs)
+        # Finally, replace this kernel node with the Assignment
+        self.replace_with(assign)
 
 
 class LFRicAPlusXKern(LFRicBuiltIn):
@@ -400,22 +476,24 @@ class LFRicAPlusXKern(LFRicBuiltIn):
     def __str__(self):
         return "Built-in: a_plus_X (real-valued fields)"
 
-    def gen_code(self, parent):
+    def lower_to_language_level(self):
         '''
-        Generates LFRic API specific PSy code for a call to the
-        a_plus_X Built-in.
-
-        :param parent: Node in f2pygen tree to which to add call.
-        :type parent: :py:class:`psyclone.f2pygen.BaseGen`
+        Lowers this LFRic-specific built-in kernel to language-level PSyIR.
+        This BuiltIn node is replaced by an Assignment node.
 
         '''
-        # We add a scalar value to each element of f1 and store the
-        # result in f2 (real-valued fields).
-        field_name2 = self.array_ref(self._arguments.args[0].proxy_name)
-        scalar_name = self._arguments.args[1].name
-        field_name1 = self.array_ref(self._arguments.args[2].proxy_name)
-        parent.add(AssignGen(parent, lhs=field_name2,
-                             rhs=scalar_name + " + " + field_name1))
+        # Get indexed references for each of the field (proxy) arguments.
+        arg_refs = self.get_indexed_field_argument_references()
+        # Get a reference for the kernel scalar argument.
+        scalar_args = self.get_scalar_argument_references()
+
+        # Create the PSyIR for the kernel:
+        #      proxy0%data(df) = ascalar + proxy1%data(df)
+        rhs = BinaryOperation.create(BinaryOperation.Operator.ADD,
+                                     scalar_args[0], arg_refs[1])
+        assign = Assignment.create(arg_refs[0], rhs)
+        # Finally, replace this kernel node with the Assignment
+        self.replace_with(assign)
 
 
 class LFRicIncAPlusXKern(LFRicBuiltIn):
@@ -426,21 +504,25 @@ class LFRicIncAPlusXKern(LFRicBuiltIn):
     def __str__(self):
         return "Built-in: inc_a_plus_X (real-valued field)"
 
-    def gen_code(self, parent):
+    def lower_to_language_level(self):
         '''
-        Generates LFRic API specific PSy code for a call to the
-        inc_a_plus_X Built-in.
-
-        :param parent: Node in f2pygen tree to which to add call.
-        :type parent: :py:class:`psyclone.f2pygen.BaseGen`
+        Lowers this LFRic-specific built-in kernel to language-level PSyIR.
+        This BuiltIn node is replaced by an Assignment node.
 
         '''
-        # We add a scalar value to each element of f1 and return the
-        # result in f1 (real-valued fields).
-        field_name = self.array_ref(self._arguments.args[1].proxy_name)
-        scalar_name = self._arguments.args[0].name
-        parent.add(AssignGen(parent, lhs=field_name,
-                             rhs=scalar_name + " + " + field_name))
+        # Get indexed references for each of the field (proxy) arguments.
+        arg_refs = self.get_indexed_field_argument_references()
+        # Get a reference for the kernel scalar argument.
+        scalar_args = self.get_scalar_argument_references()
+
+        # Create the PSyIR for the kernel:
+        #      proxy0%data(df) = ascalar + proxy0%data(df)
+        lhs = arg_refs[0]
+        rhs = BinaryOperation.create(BinaryOperation.Operator.ADD,
+                                     scalar_args[0], lhs.copy())
+        assign = Assignment.create(lhs, rhs)
+        # Finally, replace this kernel node with the Assignment
+        self.replace_with(assign)
 
 
 class LFRicAXPlusYKern(LFRicBuiltIn):
@@ -678,6 +760,63 @@ class LFRicIncXMinusYKern(LFRicBuiltIn):
                              rhs=field_name1 + " - " + field_name2))
 
 
+class LFRicAMinusXKern(LFRicBuiltIn):
+    ''' `Y = a - X` where `a` is a real scalar and `X` and `Y` are
+    real-valued fields (DoF-wise subtraction of a scalar value).
+
+    '''
+    def __str__(self):
+        return "Built-in: a_minus_X (real-valued fields)"
+
+    def lower_to_language_level(self):
+        '''
+        Lowers this LFRic-specific built-in kernel to language-level PSyIR.
+        This BuiltIn node is replaced by an Assignment node.
+
+        '''
+        # Get indexed references for each of the field (proxy) arguments.
+        arg_refs = self.get_indexed_field_argument_references()
+        # Get a reference for the kernel scalar argument.
+        scalar_args = self.get_scalar_argument_references()
+
+        # Create the PSyIR for the kernel:
+        #      proxy0%data(df) = ascalar - proxy1%data(df)
+        rhs = BinaryOperation.create(BinaryOperation.Operator.SUB,
+                                     scalar_args[0], arg_refs[1])
+        assign = Assignment.create(arg_refs[0], rhs)
+        # Finally, replace this kernel node with the Assignment
+        self.replace_with(assign)
+
+
+class LFRicIncAMinusXKern(LFRicBuiltIn):
+    ''' `X = a - X` where `a` is a real scalar and `X` is a real-valued
+    field (DoF-wise subtraction of a scalar value).
+
+    '''
+    def __str__(self):
+        return "Built-in: inc_a_minus_X (real-valued field)"
+
+    def lower_to_language_level(self):
+        '''
+        Lowers this LFRic-specific built-in kernel to language-level PSyIR.
+        This BuiltIn node is replaced by an Assignment node.
+
+        '''
+        # Get indexed references for each of the field (proxy) arguments.
+        arg_refs = self.get_indexed_field_argument_references()
+        # Get a reference for the kernel scalar argument.
+        scalar_args = self.get_scalar_argument_references()
+
+        # Create the PSyIR for the kernel:
+        #      proxy0%data(df) = ascalar - proxy0%data(df)
+        lhs = arg_refs[0]
+        rhs = BinaryOperation.create(BinaryOperation.Operator.SUB,
+                                     scalar_args[0], lhs.copy())
+        assign = Assignment.create(lhs, rhs)
+        # Finally, replace this kernel node with the Assignment
+        self.replace_with(assign)
+
+
 class LFRicAXMinusYKern(LFRicBuiltIn):
     ''' `Z = a*X - Y` where `a` is a real scalar and `Z`, `X` and
     `Y` are real-valued fields.
@@ -892,22 +1031,24 @@ class LFRicATimesXKern(LFRicBuiltIn):
     def __str__(self):
         return "Built-in: Copy a scaled real-valued field"
 
-    def gen_code(self, parent):
+    def lower_to_language_level(self):
         '''
-        Generates LFRic API specific PSy code for a call to the
-        a_times_X Built-in.
-
-        :param parent: Node in f2pygen tree to which to add call.
-        :type parent: :py:class:`psyclone.f2pygen.BaseGen`
+        Lowers this LFRic-specific built-in kernel to language-level PSyIR.
+        This BuiltIn node is replaced by an Assignment node.
 
         '''
-        # We multiply each element of f1 by the real scalar argument and
-        # store the result in f2 (real-valued fields).
-        field_name2 = self.array_ref(self._arguments.args[0].proxy_name)
-        scalar_name = self._arguments.args[1].name
-        field_name1 = self.array_ref(self._arguments.args[2].proxy_name)
-        parent.add(AssignGen(parent, lhs=field_name2,
-                             rhs=scalar_name + " * " + field_name1))
+        # Get indexed references for each of the field (proxy) arguments.
+        arg_refs = self.get_indexed_field_argument_references()
+        # Get a reference for the kernel scalar argument.
+        scalar_args = self.get_scalar_argument_references()
+
+        # Create the PSyIR for the kernel:
+        #      proxy0%data(df) = ascalar * proxy1%data(df)
+        rhs = BinaryOperation.create(BinaryOperation.Operator.MUL,
+                                     scalar_args[0], arg_refs[1])
+        assign = Assignment.create(arg_refs[0], rhs)
+        # Finally, replace this kernel node with the Assignment
+        self.replace_with(assign)
 
 
 class LFRicIncATimesXKern(LFRicBuiltIn):
@@ -917,21 +1058,25 @@ class LFRicIncATimesXKern(LFRicBuiltIn):
     def __str__(self):
         return "Built-in: Scale a real-valued field"
 
-    def gen_code(self, parent):
+    def lower_to_language_level(self):
         '''
-        Generates LFRic API specific PSy code for a call to the
-        inc_a_times_X Built-in.
-
-        :param parent: Node in f2pygen tree to which to add call.
-        :type parent: :py:class:`psyclone.f2pygen.BaseGen`
+        Lowers this LFRic-specific built-in kernel to language-level PSyIR.
+        This BuiltIn node is replaced by an Assignment node.
 
         '''
-        # In this case we're multiplying each element of a real-valued
-        # field by the supplied real scalar value.
-        field_name = self.array_ref(self._arguments.args[1].proxy_name)
-        scalar_name = self._arguments.args[0].name
-        parent.add(AssignGen(parent, lhs=field_name,
-                             rhs=scalar_name + " * " + field_name))
+        # Get indexed references for each of the field (proxy) arguments.
+        arg_refs = self.get_indexed_field_argument_references()
+        # Get a reference for the kernel scalar argument.
+        scalar_args = self.get_scalar_argument_references()
+
+        # Create the PSyIR for the kernel:
+        #      proxy0%data(df) = ascalar * proxy0%data(df)
+        lhs = arg_refs[0]
+        rhs = BinaryOperation.create(BinaryOperation.Operator.MUL,
+                                     scalar_args[0], lhs.copy())
+        assign = Assignment.create(lhs, rhs)
+        # Finally, replace this kernel node with the Assignment
+        self.replace_with(assign)
 
 
 # ------------------------------------------------------------------- #
@@ -1003,22 +1148,24 @@ class LFRicADividebyXKern(LFRicBuiltIn):
     def __str__(self):
         return "Built-in: Inverse scaling of a real-valued field (Y = a/X)"
 
-    def gen_code(self, parent):
+    def lower_to_language_level(self):
         '''
-        Generates LFRic API specific PSy code for a call to the
-        a_divideby_X Built-in.
-
-        :param parent: Node in f2pygen tree to which to add call.
-        :type parent: :py:class:`psyclone.f2pygen.BaseGen`
+        Lowers this LFRic-specific built-in kernel to language-level PSyIR.
+        This BuiltIn node is replaced by an Assignment node.
 
         '''
-        # We divide the real scalar argument by each element of f1 and
-        # store the result in f2 (real-valued fields).
-        field_name2 = self.array_ref(self._arguments.args[0].proxy_name)
-        scalar_name = self._arguments.args[1].name
-        field_name1 = self.array_ref(self._arguments.args[2].proxy_name)
-        parent.add(AssignGen(parent, lhs=field_name2,
-                             rhs=scalar_name + " / " + field_name1))
+        # Get indexed references for each of the field (proxy) arguments.
+        arg_refs = self.get_indexed_field_argument_references()
+        # Get a reference for the kernel scalar argument.
+        scalar_args = self.get_scalar_argument_references()
+
+        # Create the PSyIR for the kernel:
+        #      proxy0%data(df) = ascalar / proxy1%data(df)
+        rhs = BinaryOperation.create(BinaryOperation.Operator.DIV,
+                                     scalar_args[0], arg_refs[1])
+        assign = Assignment.create(arg_refs[0], rhs)
+        # Finally, replace this kernel node with the Assignment
+        self.replace_with(assign)
 
 
 class LFRicIncADividebyXKern(LFRicBuiltIn):
@@ -1030,21 +1177,25 @@ class LFRicIncADividebyXKern(LFRicBuiltIn):
     def __str__(self):
         return "Built-in: Inverse scaling of a real-valued field (X = a/X)"
 
-    def gen_code(self, parent):
+    def lower_to_language_level(self):
         '''
-        Generates LFRic API specific PSy code for a call to the
-        inc_a_divideby_X Built-in.
-
-        :param parent: Node in f2pygen tree to which to add call.
-        :type parent: :py:class:`psyclone.f2pygen.BaseGen`
+        Lowers this LFRic-specific built-in kernel to language-level PSyIR.
+        This BuiltIn node is replaced by an Assignment node.
 
         '''
-        # We divide the real scalar argument by each element of f1 and
-        # return the result in f1 (real-valued field).
-        field_name = self.array_ref(self._arguments.args[1].proxy_name)
-        scalar_name = self._arguments.args[0].name
-        parent.add(AssignGen(parent, lhs=field_name,
-                             rhs=scalar_name + " / " + field_name))
+        # Get indexed references for each of the field (proxy) arguments.
+        arg_refs = self.get_indexed_field_argument_references()
+        # Get a reference for the kernel scalar argument.
+        scalar_args = self.get_scalar_argument_references()
+
+        # Create the PSyIR for the kernel:
+        #      proxy0%data(df) = ascalar / proxy0%data(df)
+        lhs = arg_refs[0]
+        rhs = BinaryOperation.create(BinaryOperation.Operator.DIV,
+                                     scalar_args[0], lhs.copy())
+        assign = Assignment.create(lhs, rhs)
+        # Finally, replace this kernel node with the Assignment
+        self.replace_with(assign)
 
 
 # ------------------------------------------------------------------- #
@@ -1059,21 +1210,23 @@ class LFRicIncXPowrealAKern(LFRicBuiltIn):
     def __str__(self):
         return "Built-in: Raise a real-valued field to a real power"
 
-    def gen_code(self, parent):
+    def lower_to_language_level(self):
         '''
-        Generates LFRic API specific PSy code for a call to the
-        X_powreal_a Built-in.
-
-        :param parent: Node in f2pygen tree to which to add call.
-        :type parent: :py:class:`psyclone.f2pygen.BaseGen`
+        Lowers this LFRic-specific built-in kernel to language-level PSyIR.
+        This BuiltIn node is replaced by an Assignment node.
 
         '''
-        # In this case we're raising each element of a real-valued field
-        # to a supplied real scalar value.
-        field_name = self.array_ref(self._arguments.args[0].proxy_name)
-        real_power = self._arguments.args[1].name
-        parent.add(AssignGen(parent, lhs=field_name,
-                             rhs=field_name + "**" + real_power))
+        # Get PSyIR for each of the arguments.
+        arg_refs = self.get_indexed_field_argument_references()
+        scalar_args = self.get_scalar_argument_references()
+
+        # Create the PSyIR for the kernel:
+        #      proxy0%data(df) = proxy0%data(df) ** real_power
+        lhs = arg_refs[0]
+        rhs = BinaryOperation.create(BinaryOperation.Operator.POW,
+                                     lhs.copy(), scalar_args[0])
+        assign = Assignment.create(lhs, rhs)
+        self.replace_with(assign)
 
 
 class LFRicIncXPowintNKern(LFRicBuiltIn):
@@ -1083,21 +1236,23 @@ class LFRicIncXPowintNKern(LFRicBuiltIn):
     def __str__(self):
         return "Built-in: Raise a real-valued field to an integer power"
 
-    def gen_code(self, parent):
+    def lower_to_language_level(self):
         '''
-        Generates LFRic API specific PSy code for a call to the
-        X_powint_n Built-in.
-
-        :param parent: Node in f2pygen tree to which to add call.
-        :type parent: :py:class:`psyclone.f2pygen.BaseGen`
+        Lowers this LFRic-specific built-in kernel to language-level PSyIR.
+        This BuiltIn node is replaced by an Assignment node.
 
         '''
-        # In this case we're raising each element of a real-valued field
-        # to a supplied integer scalar value.
-        field_name = self.array_ref(self._arguments.args[0].proxy_name)
-        integer_power = self._arguments.args[1].name
-        parent.add(AssignGen(parent, lhs=field_name,
-                             rhs=field_name + "**" + integer_power))
+        # Get PSyIR for each of the arguments.
+        arg_refs = self.get_indexed_field_argument_references()
+        scalar_args = self.get_scalar_argument_references()
+
+        # Create the PSyIR for the kernel:
+        #      proxy0%data(df) = proxy0%data(df) ** int_power
+        lhs = arg_refs[0]
+        rhs = BinaryOperation.create(BinaryOperation.Operator.POW,
+                                     lhs.copy(), scalar_args[0])
+        assign = Assignment.create(lhs, rhs)
+        self.replace_with(assign)
 
 
 # ------------------------------------------------------------------- #
@@ -1382,13 +1537,37 @@ class LFRicIntIncXMinusYKern(LFRicIncXMinusYKern):
     the corresponding element of another, integer-valued, field, `X`,
     and store the result back in `X`.
     Inherits the `gen_code` method from the real-valued built-in
-    equivalent `LFRicXMinusYKern`.
+    equivalent `LFRicIncXMinusYKern`.
 
     '''
     def __str__(self):
         return "Built-in: Decrement an integer-valued field"
 
 
+class LFRicIntAMinusXKern(LFRicAMinusXKern):
+    ''' Subtract each element of an integer-valued field, `X`, from
+    an integer scalar value, `a`, and return the result as a second,
+    integer-valued, field, `Y`.
+    Inherits the `lower_to_language_level` method from the real-valued
+    built-in equivalent `LFRicAMinusXKern`.
+
+    '''
+    def __str__(self):
+        return "Built-in: int_a_minus_X (integer-valued fields)"
+
+
+class LFRicIntIncAMinusXKern(LFRicIncAMinusXKern):
+    ''' Subtract each element of an integer-valued field, `X`, from
+    an integer scalar value, `a`, and return the result in the
+    same field.
+    Inherits the `lower_to_language_level` method from the real-valued
+    built-in equivalent `LFRicIncAMinusXKern`.
+
+    '''
+    def __str__(self):
+        return "Built-in: int_inc_a_minus_X (integer-valued field)"
+
+
 # ------------------------------------------------------------------- #
 # ============== Multiplying integer fields ========================= #
 # ------------------------------------------------------------------- #
@@ -1425,7 +1604,7 @@ class LFRicIntIncXTimesYKern(LFRicIncXTimesYKern):
 
 class LFRicIntATimesXKern(LFRicATimesXKern):
     ''' Multiply each element of the first, integer-valued, field, `X`,
-     by an integer scalar, `a`, and return the result as a second,
+    by an integer scalar, `a`, and return the result as a second,
     integer-valued, field `Y` (`Y = a*X`).
     Inherits the `gen_code` method from the real-valued built-in
     equivalent `LFRicATimesXKern`.
@@ -1548,6 +1727,8 @@ REAL_BUILTIN_MAP_CAPITALISED = {
     # Subtracting (scaled) real fields
     "X_minus_Y": LFRicXMinusYKern,
     "inc_X_minus_Y": LFRicIncXMinusYKern,
+    "a_minus_X": LFRicAMinusXKern,
+    "inc_a_minus_X": LFRicIncAMinusXKern,
     "aX_minus_Y": LFRicAXMinusYKern,
     "X_minus_bY": LFRicXMinusBYKern,
     "inc_X_minus_bY": LFRicIncXMinusBYKern,
@@ -1593,6 +1774,8 @@ INT_BUILTIN_MAP_CAPITALISED = {
     # Subtracting integer fields
     "int_X_minus_Y": LFRicIntXMinusYKern,
     "int_inc_X_minus_Y": LFRicIntIncXMinusYKern,
+    "int_a_minus_X": LFRicIntAMinusXKern,
+    "int_inc_a_minus_X": LFRicIntIncAMinusXKern,
     # Multiplying (scaled) real fields
     "int_X_times_Y": LFRicIntXTimesYKern,
     "int_inc_X_times_Y": LFRicIntIncXTimesYKern,
@@ -1616,3 +1799,60 @@ BUILTIN_MAP_CAPITALISED.update(INT_BUILTIN_MAP_CAPITALISED)
 # comparison purposes. This does not enforce case sensitivity to Fortran
 # built-in names.
 BUILTIN_MAP = get_lowercase_builtin_map(BUILTIN_MAP_CAPITALISED)
+
+
+# For AutoAPI documentation generation.
+__all__ = ['LFRicBuiltInCallFactory',
+           'LFRicBuiltIn',
+           'LFRicXPlusYKern',
+           'LFRicIncXPlusYKern',
+           'LFRicAPlusXKern',
+           'LFRicIncAPlusXKern',
+           'LFRicAXPlusYKern',
+           'LFRicIncAXPlusYKern',
+           'LFRicIncXPlusBYKern',
+           'LFRicAXPlusBYKern',
+           'LFRicIncAXPlusBYKern',
+           'LFRicAXPlusAYKern',
+           'LFRicXMinusYKern',
+           'LFRicIncXMinusYKern',
+           'LFRicAMinusXKern',
+           'LFRicIncAMinusXKern',
+           'LFRicAXMinusYKern',
+           'LFRicXMinusBYKern',
+           'LFRicIncXMinusBYKern',
+           'LFRicAXMinusBYKern',
+           'LFRicXTimesYKern',
+           'LFRicIncXTimesYKern',
+           'LFRicIncAXTimesYKern',
+           'LFRicATimesXKern',
+           'LFRicIncATimesXKern',
+           'LFRicXDividebyYKern',
+           'LFRicIncXDividebyYKern',
+           'LFRicADividebyXKern',
+           'LFRicIncADividebyXKern',
+           'LFRicIncXPowrealAKern',
+           'LFRicIncXPowintNKern',
+           'LFRicSetvalCKern',
+           'LFRicSetvalXKern',
+           'LFRicXInnerproductYKern',
+           'LFRicXInnerproductXKern',
+           'LFRicSumXKern',
+           'LFRicSignXKern',
+           'LFRicIntXKern',
+           'LFRicIntXPlusYKern',
+           'LFRicIntIncXPlusYKern',
+           'LFRicIntAPlusXKern',
+           'LFRicIntIncAPlusXKern',
+           'LFRicIntXMinusYKern',
+           'LFRicIntIncXMinusYKern',
+           'LFRicIntAMinusXKern',
+           'LFRicIntIncAMinusXKern',
+           'LFRicIntXTimesYKern',
+           'LFRicIntIncXTimesYKern',
+           'LFRicIntATimesXKern',
+           'LFRicIntIncATimesXKern',
+           'LFRicIntSetvalCKern',
+           'LFRicIntSetvalXKern',
+           'LFRicIntSignXKern',
+           'LFRicRealXKern']
