@@ -1,7 +1,7 @@
 # -----------------------------------------------------------------------------
 # BSD 3-Clause License
 #
-# Copyright (c) 2019-2020, Science and Technology Facilities Council
+# Copyright (c) 2019-2021, Science and Technology Facilities Council
 # All rights reserved.
 #
 # Redistribution and use in source and binary forms, with or without
@@ -41,7 +41,8 @@ OpenCL code from PSyIR nodes.
 
 from psyclone.psyir.backend.visitor import VisitorError
 from psyclone.psyir.backend.c import CWriter
-from psyclone.psyir.symbols import ScalarType
+from psyclone.psyir.nodes import Literal
+from psyclone.psyir.symbols import ScalarType, ArrayType
 
 
 class OpenCLWriter(CWriter):
@@ -53,9 +54,8 @@ class OpenCLWriter(CWriter):
     is raised if a visitor method for a PSyIR node has not been \
     implemented, otherwise the visitor silently continues. This is an \
     optional argument which defaults to False.
-    :param indent_string: specifies what to use for indentation. This \
+    :param str indent_string: specifies what to use for indentation. This \
     is an optional argument that defaults to two spaces.
-    :type indent_string: str or NoneType
     :param int initial_indent_depth: specifies how much indentation to \
     start with. This is an optional argument that defaults to 0.
     :param int kernel_local_size: uses the given local_size when generating \
@@ -83,6 +83,27 @@ class OpenCLWriter(CWriter):
 
         self._kernels_local_size = kernels_local_size
 
+    def __call__(self, node):
+        '''This method is called when an instance of the class is called
+        directly (like a function). This implementation is known as
+        a functor. It makes sense for this class as there is only one
+        main method - the `visit` method.
+
+        # TODO #1134: This method should be removed and instead use the generic
+        # implementation, however currently OpenCL uses a DSL-level concept
+        # in a visitor (the SymbolTable.data_arguments) and this needs to
+        # be removed and PSyIR converted to language-level before calling
+        # the backend.
+
+        :param node: A PSyIR node.
+        :type node: :py:class:`psyclone.psyir.nodes.Node`
+
+        :returns: text representation of the PSyIR tree.
+        :rtype: str
+
+        '''
+        return self._visit(node)
+
     def gen_id_variable(self, symbol, dimension_index):
         '''
         Generate the declaration and initialisation of a variable identifying
@@ -127,9 +148,23 @@ class OpenCLWriter(CWriter):
 
         :returns: The OpenCL declaration of the given of the symbol.
         :rtype: str
+
+        :raises VisitorError: if an array is encountered that does not have \
+                              a lower bound of 1 for all of its dimensions.
         '''
         prefix = ""
         if symbol.shape:
+            for dim in symbol.shape:
+                if not isinstance(dim, ArrayType.ArrayBounds):
+                    continue
+                if (not isinstance(dim.lower, Literal) or
+                        dim.lower.value != "1"):
+                    raise VisitorError(
+                        "The OpenCL backend only supports arrays with a lower"
+                        " bound of 1 in each dimension. However, array '{0}' "
+                        "has a lower bound of '{1}' for dimension {2}".format(
+                            symbol.name, self._visit(dim.lower),
+                            symbol.shape.index(dim)))
             prefix += "__global "
         return prefix + super(OpenCLWriter, self).gen_declaration(symbol)
 
