@@ -43,10 +43,13 @@ back ends.
 
 import inspect
 import six
+
+from psyclone.errors import PSycloneError
 from psyclone.psyir.nodes import Node
+from psyclone.psyir.nodes.commentable_mixin import CommentableMixin
 
 
-class VisitorError(Exception):
+class VisitorError(PSycloneError):
     '''Provides a PSyclone-specific error class for errors related to a
     PSyIR visitor.
 
@@ -54,11 +57,8 @@ class VisitorError(Exception):
 
     '''
     def __init__(self, value):
-        Exception.__init__(self, value)
-        self.value = "Visitor Error: "+value
-
-    def __str__(self):
-        return str(self.value)
+        PSycloneError.__init__(self, value)
+        self.value = "Visitor Error: "+str(value)
 
 
 class PSyIRVisitor(object):
@@ -70,17 +70,18 @@ class PSyIRVisitor(object):
         is raised if a visitor method for a PSyIR node has not been \
         implemented, otherwise the visitor silently continues. This is an \
         optional argument which defaults to False.
-    :param indent_string: Specifies what to use for indentation. This \
+    :param str indent_string: Specifies what to use for indentation. This \
         is an optional argument that defaults to two spaces.
-    :type indent_string: str or NoneType
     :param int initial_indent_depth: Specifies how much indentation to \
         start with. This is an optional argument that defaults to 0.
     :param bool check_global_constraints: whether or not to validate all \
-        global constraints when walking the tree.
+        global constraints when walking the tree. Defaults to True.
 
     :raises TypeError: if any of the supplied parameters are of the wrong type.
 
     '''
+    _COMMENT_PREFIX = None
+
     def __init__(self, skip_nodes=False, indent_string="  ",
                  initial_indent_depth=0, check_global_constraints=True):
 
@@ -88,7 +89,7 @@ class PSyIRVisitor(object):
             raise TypeError(
                 "skip_nodes should be a boolean but found '{0}'."
                 "".format(type(skip_nodes).__name__))
-        if indent_string is not None and not isinstance(indent_string, str):
+        if not isinstance(indent_string, str):
             raise TypeError(
                 "indent_string should be a str but found '{0}'."
                 "".format(type(indent_string).__name__))
@@ -143,9 +144,11 @@ class PSyIRVisitor(object):
 
     def __call__(self, node):
         '''This method is called when an instance of the class is called
-        directly (like a function). This implementation is known as
-        a functor. It makes sense for this class as there is only one
-        main method - the `visit` method.
+        directly (like a function). It creates a copy of the whole tree of
+        the provided node (in order to return without any side-effects to
+        the original tree), then lower the DSL concepts into language level
+        nodes, and finally recurse down the node using the visitors defined
+        in this Visitor class.
 
         :param node: A PSyIR node.
         :type node: :py:class:`psyclone.psyir.nodes.Node`
@@ -153,8 +156,40 @@ class PSyIRVisitor(object):
         :returns: text representation of the PSyIR tree.
         :rtype: str
 
+        :raises TypeError: if the provided argument is not a PSyIR Node.
+
         '''
-        return self._visit(node)
+        if not isinstance(node, Node):
+            raise TypeError(
+                "The PSyIR visitor functor method only accepts a PSyIR Node "
+                "as argument, but found '{0}'.".format(type(node).__name__))
+
+        # The visitor must not alter the provided node but if there are any
+        # DSL concepts then these will need to be lowered in-place and this
+        # operation often modifies the tree. Therefore, we first create a
+        # copy of the full provided tree (as modifications can be above the
+        # provided node - e.g. adding a symbol in the scope)
+        tree_copy = node.root.copy()
+
+        # Get the node in the new tree with equivalent position to the provided
+        # node
+        node_copy = tree_copy.walk(Node)[node.abs_position]
+
+        # Lower the DSL concepts starting from the selected node.
+        # pylint: disable=broad-except
+        try:
+            node_copy.lower_to_language_level()
+        except Exception as error:
+            six.raise_from(VisitorError(
+                "Failed to lower '{0}'. Note that some nodes need to be "
+                "lowered from an ancestor in order to properly apply their "
+                "in-tree modifications.".format(node)), error)
+
+        # Find again the equivalent node in the lowered tree in case that it
+        # has been replaced
+        lowered_node = tree_copy.walk(Node)[node.abs_position]
+
+        return self._visit(lowered_node)
 
     def _visit(self, node):
         '''Implements the PSyIR callbacks. Callbacks are implemented by using
@@ -178,6 +213,7 @@ class PSyIRVisitor(object):
             raises an AttributeError.
 
         '''
+        # pylint: disable=too-many-branches
         if not isinstance(node, Node):
             raise VisitorError(
                 "Expected argument to be of type 'Node' but found '{0}'."
@@ -199,7 +235,38 @@ class PSyIRVisitor(object):
         for method_name in possible_method_names:
             try:
                 # pylint: disable=eval-used
-                return eval("self.{0}(node)".format(method_name))
+                node_result = eval("self.{0}(node)".format(method_name))
+
+                # We can only proceed to add comments if the Visitor
+                # returned a string, otherwise we just return
+                if not isinstance(node_result, str):
+                    return node_result
+
+                result = ""
+
+                # Add preceding comment if available
+                if isinstance(node, CommentableMixin):
+                    if node.preceding_comment and self._COMMENT_PREFIX:
+                        result += (self._nindent + self._COMMENT_PREFIX +
+                                   node.preceding_comment + "\n")
+
+                result += node_result
+
+                # Add inline comment if available
+                if isinstance(node, CommentableMixin):
+                    if node.inline_comment and self._COMMENT_PREFIX:
+                        if result[-1] != "\n":
+                            raise VisitorError(
+                                "An inline_comment can only be added to a "
+                                "construct that finishes with a '\\n', "
+                                "indicating that the line has ended, but"
+                                " node '{0}' results in '{1}'."
+                                "".format(node, result))
+                        # Add the comment before the last line break
+                        result = (result[:-1] + "  " + self._COMMENT_PREFIX +
+                                  node.inline_comment + "\n")
+
+                return result
 
             except AttributeError as excinfo:
                 if "attribute '{0}'".format(method_name) in str(excinfo):
