@@ -34,6 +34,7 @@
 # Author S. Siso, STFC Daresbury Lab
 # Modified: A. R. Porter and R. W. Ford, STFC Daresbury Lab
 # Modified: J. Henrichs, Bureau of Meteorology
+# Modified: A. B. G. Chalk, STFC Daresbury Lab
 # -----------------------------------------------------------------------------
 
 ''' PSyIR nodes package module '''
@@ -71,6 +72,19 @@ from psyclone.psyir.nodes.statement import Statement
 from psyclone.psyir.nodes.structure_reference import StructureReference
 from psyclone.psyir.nodes.structure_member import StructureMember
 from psyclone.psyir.nodes.call import Call
+from psyclone.psyir.nodes.file_container import FileContainer
+from psyclone.psyir.nodes.directive import Directive, StandaloneDirective, \
+    RegionDirective
+from psyclone.psyir.nodes.acc_directives import ACCDirective, \
+    ACCLoopDirective, ACCEnterDataDirective, ACCParallelDirective, \
+    ACCKernelsDirective, ACCDataDirective, ACCUpdateDirective, \
+    ACCStandaloneDirective, ACCRegionDirective
+from psyclone.psyir.nodes.omp_directives import OMPDirective, OMPDoDirective, \
+    OMPParallelDirective, OMPParallelDoDirective, OMPSingleDirective, \
+    OMPMasterDirective, OMPSerialDirective, OMPTaskloopDirective, \
+    OMPTaskwaitDirective, OMPStandaloneDirective, OMPRegionDirective, \
+    OMPTargetDirective, OMPLoopDirective
+
 
 # The entities in the __all__ list are made available to import directly from
 # this package e.g. 'from psyclone.psyir.nodes import Literal'
@@ -86,20 +100,15 @@ __all__ = [
         'CodeBlock',
         'Container',
         'DataNode',
-        'ExtractNode',
+        'FileContainer',
         'IfBlock',
-        'KernelSchedule',
         'Literal',
         'Loop',
         'Member',
-        'NanTestNode',
         'NaryOperation',
         'Node',
         'Operation',
-        'ProfileNode',
-        'PSyDataNode',
         'Range',
-        'ReadOnlyVerifyNode',
         'Reference',
         'Return',
         'Routine',
@@ -108,4 +117,39 @@ __all__ = [
         'StructureMember',
         'StructureReference',
         'UnaryOperation',
-        'ScopingNode']
+        'ScopingNode',
+        # PSyclone-specific nodes
+        'KernelSchedule',
+        # PSyData Nodes
+        'PSyDataNode',
+        'ExtractNode',
+        'ProfileNode',
+        'NanTestNode',
+        'ReadOnlyVerifyNode',
+        # Directive Nodes
+        'Directive',
+        'RegionDirective',
+        'StandaloneDirective',
+        'ACCDirective',
+        'ACCRegionDirective',
+        'ACCStandaloneDirective',
+        'ACCDataDirective',
+        'ACCEnterDataDirective',
+        'ACCParallelDirective',
+        'ACCLoopDirective',
+        'ACCKernelsDirective',
+        'ACCUpdateDirective',
+        'OMPDirective',
+        'OMPRegionDirective',
+        'OMPStandaloneDirective',
+        'OMPParallelDirective',
+        'OMPSerialDirective',
+        'OMPSingleDirective',
+        'OMPMasterDirective',
+        'OMPTaskloopDirective',
+        'OMPDoDirective',
+        'OMPParallelDoDirective',
+        'OMPTaskwaitDirective',
+        'OMPTargetDirective',
+        'OMPLoopDirective'
+        ]
