@@ -43,14 +43,15 @@ from psyclone.psyir.nodes.routine import Routine
 from psyclone.psyir.nodes.codeblock import CodeBlock
 from psyclone.psyir.symbols import SymbolTable
 from psyclone.errors import GenerationError
+from psyclone.psyir.nodes.commentable_mixin import CommentableMixin
 
 
-class Container(ScopingNode):
-    '''Node representing a set of KernelSchedule and/or Container nodes,
-    as well as a name and a SymbolTable. This construct can be used to
-    scope symbols of variables, KernelSchedule names and Container
-    names. In Fortran a container would naturally represent a module
-    or a submodule.
+class Container(ScopingNode, CommentableMixin):
+    '''Node representing a set of Routine and/or Container nodes, as well
+    as a name and a SymbolTable. This construct can be used to scope
+    symbols of variables, Routine names and Container names. In
+    Fortran a container would naturally represent a module or a
+    submodule.
 
     :param str name: the name of the container.
     :param parent: optional parent node of this Container in the PSyIR.
@@ -84,8 +85,8 @@ class Container(ScopingNode):
         # pylint: disable=unused-argument
         return isinstance(child, (Container, Routine, CodeBlock))
 
-    @staticmethod
-    def create(name, symbol_table, children):
+    @classmethod
+    def create(cls, name, symbol_table, children):
         '''Create a Container instance given a name, a symbol table and a
         list of child nodes.
 
@@ -94,12 +95,13 @@ class Container(ScopingNode):
             Container.
         :type symbol_table: :py:class:`psyclone.psyir.symbols.SymbolTable`
         :param children: a list of PSyIR nodes contained in the \
-            Container. These must be Containers or KernelSchedules.
+            Container. These must be Containers or Routines.
         :type children: list of :py:class:`psyclone.psyir.nodes.Container` \
-            or :py:class:`psyclone.psyir.nodes.KernelSchedule`
+            or :py:class:`psyclone.psyir.nodes.Routine`
 
-        :returns: a Container instance.
-        :rtype: :py:class:`psyclone.psyir.nodes.Container`
+        :returns: an instance of `cls`.
+        :rtype: :py:class:`psyclone.psyir.nodes.Container` or subclass
+            thereof
 
         :raises GenerationError: if the arguments to the create method \
             are not of the expected type.
@@ -121,7 +123,7 @@ class Container(ScopingNode):
                 "should be a list but found '{0}'."
                 "".format(type(children).__name__))
 
-        container = Container(name)
+        container = cls(name)
         # pylint: disable=protected-access
         container._symbol_table = symbol_table
         symbol_table._node = container
