@@ -40,10 +40,13 @@
 This module contains the abstract Node implementation.
 
 '''
+from __future__ import print_function
+
 import copy
 import six
-from psyclone.psyir.symbols import SymbolError
+
 from psyclone.errors import GenerationError, InternalError
+from psyclone.psyir.symbols import SymbolError
 
 # Default indentation string
 INDENTATION_STRING = "    "
@@ -403,10 +406,9 @@ class Node(object):
         :rtype: str
         '''
         if not self._text_name:
-            raise NotImplementedError(
-                "_text_name is an abstract attribute which needs to be "
-                "given a string value in the concrete class '{0}'."
-                "".format(type(self).__name__))
+            name_string = type(self).__name__
+        else:
+            name_string = self._text_name
         if colour:
             if self._colour is None:
                 raise NotImplementedError(
@@ -414,14 +416,14 @@ class Node(object):
                     "a string value in the concrete class '{0}'."
                     "".format(type(self).__name__))
             try:
-                return colored(self._text_name, self._colour)
+                return colored(name_string, self._colour)
             except KeyError as info:
                 message = (
                     "The _colour attribute in class '{0}' has been set to a "
                     "colour ('{1}') that is not supported by the termcolor "
                     "package.".format(type(self).__name__, self._colour))
                 six.raise_from(InternalError(message), info)
-        return self._text_name
+        return name_string
 
     def node_str(self, colour=True):
         '''
@@ -546,8 +548,11 @@ class Node(object):
         edges (but their direction is reversed so the layout looks
         reasonable) and parent child dependencies are represented as
         blue edges.'''
-        # pylint: disable=too-many-branches
+        # Import outside top-level to avoid circular dependencies.
+        # pylint: disable=too-many-branches, import-outside-toplevel
         from psyclone.psyir.nodes.loop import Loop
+        from psyclone.psyir.nodes.routine import Routine
+
         # names to append to my default name to create start and end vertices
         start_postfix = "_start"
         end_postfix = "_end"
@@ -573,9 +578,9 @@ class Node(object):
                 remote_name += start_postfix
             # Create the forward dependence edge in green
             graph.edge(local_name, remote_name, color="green")
-        elif self.parent:
-            # this node is a child of another node and has no forward
-            # dependence. Therefore connect it to the the end vertex
+        elif not isinstance(self, Routine):
+            # If this node is not a Routine (where the DAG context finishes)
+            # and has no forward dependence, connect it to the end vertex
             # of its parent. Use blue to indicate a parent child
             # relationship.
             remote_name = self.parent.dag_name + end_postfix
@@ -599,9 +604,9 @@ class Node(object):
                 remote_name += end_postfix
             # Create the backward dependence edge in red.
             graph.edge(remote_name, local_name, color="red")
-        elif self.parent:
-            # this node has a parent and has no backward
-            # dependence. Therefore connect it to the the start vertex
+        elif not isinstance(self, Routine):
+            # If this node is not a Routine (where the DAG context finishes)
+            # and has no backward dependence, connect it to the start vertex
             # of its parent. Use blue to indicate a parent child
             # relationship.
             remote_name = self.parent.dag_name + start_postfix
@@ -618,8 +623,23 @@ class Node(object):
 
     @property
     def dag_name(self):
-        '''Return the base dag name for this node.'''
-        return "node_" + str(self.abs_position)
+        '''Return the dag name for this node. This includes the name of the
+        class and the index of its relative position to the parent Routine. If
+        no parent Routine is found, the index used is the absolute position
+        in the tree.
+
+        :returns: the dag name for this node.
+        :rtype: str
+
+        '''
+        # Import here to avoid circular dependencies
+        # pylint: disable=import-outside-toplevel
+        from psyclone.psyir.nodes import Routine
+        if self.ancestor(Routine):
+            _, position = self._find_position(self.ancestor(Routine))
+        else:
+            position = self.abs_position
+        return self.coloured_name(False) + "_" + str(position)
 
     @property
     def args(self):
@@ -656,9 +676,9 @@ class Node(object):
                 # the tree as me.
                 while node.depth > self.depth:
                     node = node.parent
-                if self.sameParent(node):
+                if self.sameParent(node) and node is not self:
                     # The remote node (or one of its ancestors) shares
-                    # the same parent as me
+                    # the same parent as me (but its not me)
                     if not dependence:
                         # this is the first dependence found so keep it
                         dependence = node
@@ -693,9 +713,9 @@ class Node(object):
                 # the tree as me.
                 while node.depth > self.depth:
                     node = node.parent
-                if self.sameParent(node):
+                if self.sameParent(node) and node is not self:
                     # The remote node (or one of its ancestors) shares
-                    # the same parent as me
+                    # the same parent as me (but its not me)
                     if not dependence:
                         # this is the first dependence found so keep it
                         dependence = node
@@ -807,11 +827,12 @@ class Node(object):
         '''
         from psyclone.psyir.nodes import Schedule
         if not isinstance(self.parent, Schedule) or index is None:
-            print("{0}{1}".format(self.indent(indent),
-                                  self.node_str(colour=True)))
+            result = "{0}{1}".format(self.indent(indent),
+                                     self.node_str(colour=True))
         else:
-            print("{0}{1}: {2}".format(self.indent(indent), index,
-                                       self.node_str(colour=True)))
+            result = "{0}{1}: {2}".format(self.indent(indent), index,
+                                          self.node_str(colour=True))
+        print(six.text_type(result))
         for idx, entity in enumerate(self._children):
             entity.view(indent=indent + 1, index=idx)
 
@@ -898,13 +919,13 @@ class Node(object):
         it is the root). Needs to be computed dynamically from the
         starting position (0) as its position may change.
 
-        :returns: absolute position of a Node in the tree
+        :returns: absolute position of a Node in the tree.
         :rtype: int
 
-        :raises InternalError: if the absolute position cannot be found
+        :raises InternalError: if the absolute position cannot be found.
+
         '''
-        from psyclone.psyir.nodes import Schedule
-        if self.root == self and isinstance(self.root, Schedule):
+        if self.root == self:
             return self.START_POSITION
         found, position = self._find_position(self.root.children,
                                               self.START_POSITION)
@@ -913,17 +934,26 @@ class Node(object):
                                 "in the tree")
         return position
 
-    def _find_position(self, children, position):
+    def _find_position(self, children, position=None):
         '''
         Recurse through the tree depth first returning position of
         a Node if found.
-        :param children: list of Nodes which are children of this Node
+
+        :param children: list of Nodes which are children of this Node.
         :type children: list of :py:class:`psyclone.psyir.nodes.Node`
-        :returns: position of the Node in the tree
-        :rtype: int
-        :raises InternalError: if the starting position is < 0
+        :param int position: start counting from this position. Defaults to \
+            START_POSITION.
+
+        :returns: if the self has been found in the provided children list \
+            and its relative position.
+        :rtype: bool, int
+
+        :raises InternalError: if the starting position is < 0.
+
         '''
-        if position < self.START_POSITION:
+        if position is None:
+            position = self.START_POSITION
+        elif position < self.START_POSITION:
             raise InternalError(
                 "Search for Node position started from {0} "
                 "instead of {1}.".format(position, self.START_POSITION))
@@ -1055,31 +1085,66 @@ class Node(object):
         from psyclone.psyGen import Kern
         return self.walk(Kern)
 
-    def following(self):
-        '''Return all :py:class:`psyclone.psyir.nodes.Node` nodes after me in
-        the schedule. Ordering is depth first.
+    def following(self, routine=True):
+        '''Return all :py:class:`psyclone.psyir.nodes.Node` nodes after this
+        node. Ordering is depth first. If the `routine` argument is
+        set to `True` then nodes are only returned if they are
+        descendents of this node's closest ancestor routine if one
+        exists.
 
-        :returns: a list of nodes
+        :param bool routine: an optional (default `True`) argument \
+            that only returns nodes that are within this node's \
+            closest ancestor Routine node if one exists.
+
+        :returns: a list of nodes.
         :rtype: :func:`list` of :py:class:`psyclone.psyir.nodes.Node`
 
         '''
-        all_nodes = self.root.walk(Node)
+        root = self.root
+        if routine:
+            # Import here to avoid circular dependencies
+            # pylint: disable=import-outside-toplevel
+            from psyclone.psyir.nodes import Routine
+            # If there is an ancestor Routine node then only return nodes
+            # that are within it.
+            routine_node = self.ancestor(Routine)
+            if routine_node:
+                root = routine_node
+        all_nodes = root.walk(Node)
         position = all_nodes.index(self)
         return all_nodes[position+1:]
 
-    def preceding(self, reverse=None):
-        '''Return all :py:class:`psyclone.psyir.nodes.Node` nodes before me
-        in the schedule. Ordering is depth first. If the `reverse` argument
-        is set to `True` then the node ordering is reversed
-        i.e. returning the nodes closest to me first
+    def preceding(self, reverse=False, routine=True):
+        '''Return all :py:class:`psyclone.psyir.nodes.Node` nodes before this
+        node. Ordering is depth first. If the `reverse` argument is
+        set to `True` then the node ordering is reversed
+        i.e. returning the nodes closest to this node first. if the
+        `routine` argument is set to `True` then nodes are only
+        returned if they are descendents of this node's closest
+        ancestor routine if one exists.
 
-        :param: reverse: An optional, default `False`, boolean flag
-        :type: reverse: bool
-        :returns: A list of nodes
+        :param bool reverse: an optional (default `False`) argument \
+            that reverses the order of any returned nodes (i.e. makes \
+            them 'closest first' if set to true.
+        :param bool routine: an optional (default `True`) argument \
+            that only returns nodes that are within this node's \
+            closest ancestor Routine node if one exists.
+
+        :returns: a list of nodes.
         :rtype: :func:`list` of :py:class:`psyclone.psyir.nodes.Node`
 
         '''
-        all_nodes = self.root.walk(Node)
+        root = self.root
+        if routine:
+            # Import here to avoid circular dependencies
+            # pylint: disable=import-outside-toplevel
+            from psyclone.psyir.nodes import Routine
+            # If there is an ancestor Routine node then only return nodes
+            # that are within it.
+            routine_node = self.ancestor(Routine)
+            if routine_node:
+                root = routine_node
+        all_nodes = root.walk(Node)
         position = all_nodes.index(self)
         nodes = all_nodes[:position]
         if reverse:
@@ -1127,7 +1192,7 @@ class Node(object):
         ''':returns: True if this Node is within an OpenMP parallel region.
 
         '''
-        from psyclone.psyGen import OMPParallelDirective
+        from psyclone.psyir.nodes import OMPParallelDirective
         omp_dir = self.ancestor(OMPParallelDirective)
         if omp_dir:
             return True
@@ -1141,16 +1206,12 @@ class Node(object):
         that represent high-level concepts.
 
         '''
-        for child in self.children:
+        # We recurse only over the original children (hence [:]), this is
+        # because new nodes may be inserted during the lowering, but these
+        # must already be language-level.
+        for child in self.children[:]:
             child.lower_to_language_level()
 
-    def update(self):
-        ''' By default we assume there is no need to update the existing
-        fparser2 AST which this Node represents. We simply call the update()
-        method of any children. '''
-        for child in self._children:
-            child.update()
-
     def reference_accesses(self, var_accesses):
         '''Get all variable access information. The default implementation
         just recurses down to all children.
@@ -1162,31 +1223,6 @@ class Node(object):
         for child in self._children:
             child.reference_accesses(var_accesses)
 
-    def _insert_schedule(self, children=None, ast=None):
-        '''
-        Utility method to insert a Schedule between this Node and the
-        supplied list of children.
-
-        :param children: nodes which will become children of the \
-                         new Schedule.
-        :type children: list of :py:class:`psyclone.psyir.nodes.Node`
-        :param ast: reference to fparser2 parse tree for associated \
-                    Fortran code.
-        :type ast: :py:class:`fparser.two.utils.Base`
-
-        :returns: the new Schedule node.
-        :rtype: :py:class:`psyclone.psyir.nodes.Schedule`
-        '''
-        from psyclone.psyir.nodes import Schedule
-        sched = Schedule(children=children, parent=self)
-        if children:
-            # If we have children then set the Schedule's AST pointer to
-            # point to the AST associated with them.
-            sched.ast = children[0].ast
-        else:
-            sched.ast = ast
-        return sched
-
     @property
     def scope(self):
         ''' Some nodes (e.g. Schedule and Container) allow symbols to be
