@@ -66,14 +66,6 @@ class Schedule(ScopingNode):
         # pylint: disable=unused-argument
         return isinstance(child, Statement)
 
-    @property
-    def dag_name(self):
-        '''
-        :returns: The name of this node in the dag.
-        :rtype: str
-        '''
-        return "schedule_" + str(self.abs_position)
-
     def __getitem__(self, index):
         '''
         Overload the subscript notation ([int]) to access specific statements
