@@ -42,7 +42,8 @@ from collections import OrderedDict, namedtuple
 from enum import Enum
 import six
 from psyclone.errors import InternalError
-from psyclone.psyir.symbols import TypeSymbol, DataSymbol
+from psyclone.psyir.symbols.data_type_symbol import DataTypeSymbol
+from psyclone.psyir.symbols.datasymbol import DataSymbol
 from psyclone.psyir.symbols.symbol import Symbol
 
 
@@ -71,6 +72,14 @@ class DeferredType(DataType):
         return "DeferredType"
 
 
+class NoType(DataType):
+    ''' Indicates that the associated symbol has an empty type (equivalent
+    to `void` in C). '''
+
+    def __str__(self):
+        return "NoType"
+
+
 @six.add_metaclass(abc.ABCMeta)
 class UnknownType(DataType):
     '''
@@ -226,27 +235,29 @@ class ScalarType(DataType):
 class ArrayType(DataType):
     '''Describes an array datatype. Can be an array of intrinsic types (e.g.
     integer) or of structure types. For the latter, the type must currently be
-    specified as a TypeSymbol (see #1031).
+    specified as a DataTypeSymbol (see #1031).
 
     :param datatype: the datatype of the array elements.
     :type datatype: :py:class:`psyclone.psyir.datatypes.DataType` or \
-        :py:class:`psyclone.psyir.symbols.TypeSymbol`
-    :param list shape: shape of the symbol in column-major order \
-        (leftmost index is contiguous in memory). Each entry \
-        represents an array dimension. If it is \
-        ArrayType.Extent.ATTRIBUTE the extent of that dimension is \
-        unknown but can be obtained by querying the run-time system \
-        (e.g. using the SIZE intrinsic in Fortran). If it is \
-        ArrayType.Extent.DEFERRED then the extent is also unknown and \
-        may or may not be defined at run-time (e.g. the array is \
-        ALLOCATABLE in Fortran). Otherwise it can be an int or a \
-        DataNode (that returns an int). Note that providing an int is \
-        supported as a convenience, the provided value will be stored \
-        internally as a Literal node.
+        :py:class:`psyclone.psyir.symbols.DataTypeSymbol`
+    :param list shape: shape of the symbol in column-major order (leftmost \
+        index is contiguous in memory). Each entry represents an array \
+        dimension. If it is ArrayType.Extent.ATTRIBUTE the extent of that \
+        dimension is unknown but can be obtained by querying the run-time \
+        system (e.g. using the SIZE intrinsic in Fortran). If it is \
+        ArrayType.Extent.DEFERRED then the extent is also unknown and may or \
+        may not be defined at run-time (e.g. the array is ALLOCATABLE in \
+        Fortran). Otherwise it can be an int or a DataNode (that returns an \
+        int) or a 2-tuple of such quantities. If only a single value is \
+        provided then that is taken to be the upper bound and the lower bound \
+        defaults to 1. If a 2-tuple is provided then the two members specify \
+        the lower and upper bounds, respectively, of the current dimension. \
+        Note that providing an int is supported as a convenience, the provided\
+        value will be stored internally as a Literal node.
 
     :raises TypeError: if the arguments are of the wrong type.
     :raises NotImplementedError: if a structure type does not have a \
-                                 TypeSymbol as its type.
+                                 DataTypeSymbol as its type.
     '''
     class Extent(Enum):
         '''
@@ -260,38 +271,66 @@ class ArrayType(DataType):
         DEFERRED = 1
         ATTRIBUTE = 2
 
+    #: namedtuple used to store lower and upper limits of an array dimension
+    ArrayBounds = namedtuple("ArrayBounds", ["lower", "upper"])
+
     def __init__(self, datatype, shape):
 
-        # This import must be placed here to avoid circular
-        # dependencies.
+        # This import must be placed here to avoid circular dependencies.
         # pylint: disable=import-outside-toplevel
-        from psyclone.psyir.nodes import Literal
+        from psyclone.psyir.nodes import Literal, DataNode
+
+        def _node_from_int(var):
+            ''' Helper routine that simply creates a Literal out of an int.
+            If the supplied arg is not an int then it is returned unchanged.
+
+            :param var: variable for which to create a Literal if necessary.
+            :type var: int or :py:class:`psyclone.psyir.nodes.DataNode`
+
+            :returns: a DataNode representing the supplied input.
+            :rtype: :py:class:`psyclone.psyir.nodes.DataNode`
+
+            '''
+            if isinstance(var, int):
+                return Literal(str(var), INTEGER_TYPE)
+            return var
 
         if isinstance(datatype, DataType):
             if isinstance(datatype, StructureType):
                 # TODO #1031 remove this restriction.
                 raise NotImplementedError(
                     "When creating an array of structures, the type of "
-                    "those structures must be supplied as a TypeSymbol but "
-                    "got a StructureType instead.")
+                    "those structures must be supplied as a DataTypeSymbol "
+                    "but got a StructureType instead.")
             self._intrinsic = datatype.intrinsic
             self._precision = datatype.precision
-        elif isinstance(datatype, TypeSymbol):
+        elif isinstance(datatype, DataTypeSymbol):
             self._intrinsic = datatype
             self._precision = None
         else:
             raise TypeError(
                 "ArrayType expected 'datatype' argument to be of type "
-                "DataType or TypeSymbol but found '{0}'."
+                "DataType or DataTypeSymbol but found '{0}'."
                 "".format(type(datatype).__name__))
         # We do not have a setter for shape as it is an immutable property,
         # therefore we have a separate validation routine.
         self._validate_shape(shape)
-        # Replace any ints in shape with a Literal. int's are only
-        # supported as they allow a more concise dimension
-        # declaration.
-        self._shape = [Literal(str(dim), INTEGER_TYPE)
-                       if isinstance(dim, int) else dim for dim in shape]
+        # Replace any ints in shape with a Literal. int's are only supported
+        # as they allow a more concise dimension declaration.
+        self._shape = []
+        one = Literal("1", INTEGER_TYPE)
+        for dim in shape:
+            if isinstance(dim, (DataNode, int)):
+                # The lower bound is 1 by default.
+                self._shape.append(ArrayType.ArrayBounds(one.copy(),
+                                                         _node_from_int(dim)))
+            elif isinstance(dim, tuple):
+                self._shape.append(
+                    ArrayType.ArrayBounds(_node_from_int(dim[0]),
+                                          _node_from_int(dim[1])))
+            else:
+                self._shape.append(dim)
+
         self._datatype = datatype
 
     @property
@@ -340,7 +379,8 @@ class ArrayType(DataType):
         :param extents: list of extents, one for each array dimension.
         :type extents: list of \
             :py:class:`psyclone.psyir.symbols.ArrayType.Extent`, int \
-            or subclass of :py:class:`psyclone.psyir.nodes.DataNode`
+            or subclass of :py:class:`psyclone.psyir.nodes.DataNode` or \
+            2-tuple of int or `DataNode`.
 
         :raises TypeError: if extents is not a list.
         :raises TypeError: if one or more of the supplied extents is a \
@@ -353,6 +393,40 @@ class ArrayType(DataType):
         # dependencies.
         # pylint: disable=import-outside-toplevel
         from psyclone.psyir.nodes import DataNode, Reference
+
+        def _validate_data_node(dim_node):
+            '''
+            Checks that the supplied DataNode is valid as part of an
+            array-shape specification.
+
+            :param dim_node: the DataNode to check.
+            :type dim_node: :py:class:`psyclone.psyir.nodes.DataNode`
+
+            :raises TypeError: if the DataNode is not valid in this context.
+
+            '''
+            # When issue #685 is addressed then check that the
+            # datatype returned is an int (or is unknown). For the
+            # moment, just check that if the DataNode is a
+            # Reference then the associated symbol is a scalar
+            # integer or is unknown.
+            if isinstance(dim_node, Reference):
+                # Check the DataSymbol instance is a scalar
+                # integer or is unknown
+                symbol = dim_node.symbol
+                if not ((symbol.is_scalar and symbol.datatype.intrinsic ==
+                         ScalarType.Intrinsic.INTEGER) or
+                        isinstance(symbol.datatype,
+                                   (UnknownFortranType, DeferredType))):
+                    raise TypeError(
+                        "If a DataSymbol is referenced in a dimension "
+                        "declaration then it should be a scalar integer or "
+                        "of UnknownType or DeferredType, but '{0}' is a "
+                        "'{1}'.".format(symbol.name, symbol.datatype))
+                # TODO #1089 - add check that any References are not to a
+                # local datasymbol that is not constant (as this would have
+                # no value).
+
         if not isinstance(extents, list):
             raise TypeError(
                 "ArrayType 'shape' must be of type list but found '{0}'."
@@ -360,56 +434,64 @@ class ArrayType(DataType):
 
         for dimension in extents:
             if isinstance(dimension, DataNode):
-                # When issue #685 is addressed then check that the
-                # datatype returned is an int (or is unknown). For the
-                # moment, just check that if the DataNode is a
-                # Reference then the associated symbol is a scalar
-                # integer or is unknown.
-                if isinstance(dimension, Reference):
-                    # Check the DataSymbol instance is a scalar
-                    # integer or is unknown
-                    symbol = dimension.symbol
-                    if not ((symbol.is_scalar and symbol.datatype.intrinsic ==
-                             ScalarType.Intrinsic.INTEGER) or
-                            isinstance(symbol.datatype,
-                                       (UnknownFortranType, DeferredType))):
+                _validate_data_node(dimension)
+            elif isinstance(dimension, tuple):
+                if len(dimension) != 2:
+                    raise TypeError(
+                        "A DataSymbol shape-list element specifying lower "
+                        "and upper bounds must be a 2-tuple but '{0}' has {1} "
+                        "entries.".format(str(dimension), len(dimension)))
+                for dim in dimension:
+                    if isinstance(dim, DataNode):
+                        _validate_data_node(dim)
+                    elif not isinstance(dim, int):
                         raise TypeError(
-                            "If a datasymbol is used as a dimension "
-                            "declaration then it should be a scalar "
-                            "integer or an unknown type, but '{0}' is a "
-                            "'{1}'.".format(symbol.name, symbol.datatype))
-                # TODO #1089 - add check that any References are not to a
-                # local datasymbol that is not constant (as this would have
-                # no value).
+                            "A DataSymbol shape-list element specifying lower "
+                            "and upper bounds must be a 2-tuple containing "
+                            "either int or DataNode entries but '{0}' contains"
+                            " '{1}'".format(str(dimension),
+                                            type(dim).__name__))
             elif not isinstance(dimension, (self.Extent, int)):
                 raise TypeError(
-                    "DataSymbol shape list elements can only be "
-                    "'int', ArrayType.Extent or 'DataNode' but found "
+                    "DataSymbol shape list elements can only be 'int', "
+                    "ArrayType.Extent, 'DataNode' or tuple but found "
                     "'{0}'.".format(type(dimension).__name__))
 
     def __str__(self):
         '''
-        :returns: a description of this array datatype.
+        :returns: a description of this array datatype. If the lower bound \
+            of any dimension has the default value of 1 then it is omitted \
+            for the sake of brevity.
         :rtype: str
 
-        :raises InternalError: if an unsupported dimensions type is \
-            found.
+        :raises InternalError: if an unsupported dimensions type is found.
 
         '''
-        # This import must be placed here to avoid circular
-        # dependencies.
-        # pylint: disable=import-outside-toplevel
-        from psyclone.psyir.nodes import DataNode
         dims = []
         for dimension in self.shape:
-            if isinstance(dimension, DataNode):
-                dims.append(str(dimension))
+            if isinstance(dimension, ArrayType.ArrayBounds):
+                dim_text = ""
+                # Have to import locally to avoid circular dependence
+                # pylint: disable=import-outside-toplevel
+                from psyclone.psyir.nodes import Literal
+                # Lower bound. If it is "1" then we omit it.
+                if isinstance(dimension.lower, Literal):
+                    if dimension.lower.value != "1":
+                        dim_text = dimension.lower.value + ":"
+                else:
+                    dim_text = str(dimension.lower) + ":"
+                # Upper bound.
+                if isinstance(dimension.upper, Literal):
+                    dim_text += dimension.upper.value
+                else:
+                    dim_text += str(dimension.upper)
+                dims.append(dim_text)
             elif isinstance(dimension, ArrayType.Extent):
                 dims.append("'{0}'".format(dimension.name))
             else:
                 raise InternalError(
-                    "ArrayType shape list elements can only be 'DataNode', "
-                    "or 'ArrayType.Extent', but found '{0}'."
+                    "ArrayType shape list elements can only be 'ArrayType."
+                    "ArrayBounds', or 'ArrayType.Extent', but found '{0}'."
                     "".format(type(dimension).__name__))
         return ("Array<{0}, shape=[{1}]>".format(
             self._datatype, ", ".join(dims)))
@@ -477,7 +559,7 @@ class StructureType(DataType):
         :param str name: the name of the new component.
         :param datatype: the type of the new component.
         :type datatype: :py:class:`psyclone.psyir.symbols.DataType` or \
-                        :py:class:`psyclone.psyir.symbols.TypeSymbol`
+                        :py:class:`psyclone.psyir.symbols.DataTypeSymbol`
         :param visibility: whether this component is public or private.
         :type visibility: :py:class:`psyclone.psyir.symbols.Symbol.Visibility`
 
@@ -488,10 +570,10 @@ class StructureType(DataType):
             raise TypeError(
                 "The name of a component of a StructureType must be a 'str' "
                 "but got '{0}'".format(type(name).__name__))
-        if not isinstance(datatype, (DataType, TypeSymbol)):
+        if not isinstance(datatype, (DataType, DataTypeSymbol)):
             raise TypeError(
                 "The type of a component of a StructureType must be a "
-                "'DataType' or 'TypeSymbol' but got '{0}'".format(
+                "'DataType' or 'DataTypeSymbol' but got '{0}'".format(
                     type(datatype).__name__))
         if not isinstance(visibility, Symbol.Visibility):
             raise TypeError(
