@@ -44,9 +44,10 @@ class and its subclasses and the generic Symbol class.
 from __future__ import absolute_import
 from enum import Enum
 import six
+from psyclone.errors import PSycloneError, InternalError
 
 
-class SymbolError(Exception):
+class SymbolError(PSycloneError):
     '''
     PSyclone-specific exception for use with errors relating to the Symbol and
     SymbolTable in the PSyIR.
@@ -54,18 +55,16 @@ class SymbolError(Exception):
     :param str value: the message associated with the error.
     '''
     def __init__(self, value):
-        Exception.__init__(self, value)
-        self.value = "PSyclone SymbolTable error: "+value
-
-    def __str__(self):
-        return str(self.value)
+        PSycloneError.__init__(self, value)
+        self.value = "PSyclone SymbolTable error: "+str(value)
 
 
-class SymbolInterface(object):
+class SymbolInterface(object):   # pylint: disable=too-few-public-methods
     ''' Abstract class of a Symbol Interface '''
 
 
 class LocalInterface(SymbolInterface):
+    # pylint: disable=too-few-public-methods
     ''' The symbol just exists in the Local context '''
 
     def __str__(self):
@@ -73,16 +72,16 @@ class LocalInterface(SymbolInterface):
 
 
 class UnresolvedInterface(SymbolInterface):
+    # pylint: disable=too-few-public-methods
     '''We have a symbol but we don't know where it is declared.'''
 
     def __str__(self):
         return "Unresolved"
 
 
-class GlobalInterface(SymbolInterface):
-    '''Describes the interface to a Symbol that is supplied externally to
-    this container, and therefore it is defined in an external PSyIR
-    container.
+class ImportInterface(SymbolInterface):
+    '''Describes the interface to a Symbol that is imported from an external
+    PSyIR container.
 
     :param container_symbol: symbol representing the external container \
         from which the symbol is imported.
@@ -93,13 +92,15 @@ class GlobalInterface(SymbolInterface):
 
     '''
     def __init__(self, container_symbol):
+        # Avoid circular import
+        # pylint: disable=import-outside-toplevel
         from psyclone.psyir.symbols import ContainerSymbol
 
-        super(GlobalInterface, self).__init__()
+        super(ImportInterface, self).__init__()
 
         if not isinstance(container_symbol, ContainerSymbol):
             raise TypeError(
-                "Global container_symbol parameter must be of type"
+                "ImportInterface container_symbol parameter must be of type"
                 " ContainerSymbol, but found '{0}'."
                 "".format(type(container_symbol).__name__))
 
@@ -114,7 +115,7 @@ class GlobalInterface(SymbolInterface):
         return self._container_symbol
 
     def __str__(self):
-        return "Global(container='{0}')".format(self.container_symbol.name)
+        return "Import(container='{0}')".format(self.container_symbol.name)
 
 
 class ArgumentInterface(SymbolInterface):
@@ -182,11 +183,11 @@ class ArgumentInterface(SymbolInterface):
 
 
 class Symbol(object):
-    '''Generic Symbol item for the Symbol Table. It always has a fixed
-    name label that matches with the key in the SymbolTables that
-    contain the symbol.  If the symbol is not public then it is only
-    visible to those nodes that are descendents of the Node to which
-    its containing Symbol Table belongs.
+    '''Generic Symbol item for the Symbol Table and PSyIR References.
+    It has an immutable name label because it must always match with the
+    key in the SymbolTable. If the symbol is private then it is only visible
+    to those nodes that are descendants of the Node to which its containing
+    Symbol Table belongs.
 
     :param str name: name of the symbol.
     :param visibility: the visibility of the symbol.
@@ -197,8 +198,7 @@ class Symbol(object):
         :py:class:`psyclone.psyir.symbols.LocalInterface`
     :type interface: :py:class:`psyclone.psyir.symbols.symbol.SymbolInterface`
 
-    :raises TypeError: if the name is not a str or visibility is not an \
-                       instance of Visibility.
+    :raises TypeError: if the name is not a str.
 
     '''
 
@@ -232,13 +232,30 @@ class Symbol(object):
         # The following attributes have a setter method (with error checking)
         self._visibility = None
         self._interface = None
-        # If an interface is not provided, use LocalInterface by default
-        if not interface:
-            self._interface = LocalInterface()
-        else:
-            # Use the setter as it checks the variables validity
+
+        self._process_arguments(visibility=visibility, interface=interface)
+
+    def _process_arguments(self, visibility=None, interface=None):
+        ''' Process the visibility and interface arguments of the constructor
+        and the specialise methods.
+
+        :param visibility: the visibility of the symbol.
+        :type visibility: :py:class:`psyclone.psyir.symbols.Symbol.Visibility`
+        :param interface: optional object describing the interface to this \
+            symbol (i.e. whether it is passed as a routine argument or \
+            accessed in some other way). Defaults to \
+            :py:class:`psyclone.psyir.symbols.LocalInterface`
+        :type interface: \
+            :py:class:`psyclone.psyir.symbols.symbol.SymbolInterface`
+
+        '''
+        if interface:
             self.interface = interface
-        self.visibility = visibility
+        elif not self.interface:
+            self.interface = LocalInterface()
+
+        if visibility:
+            self.visibility = visibility
 
     def copy(self):
         '''Create and return a copy of this object. Any references to the
@@ -255,7 +272,22 @@ class Symbol(object):
         return type(self)(self.name, visibility=self.visibility,
                           interface=self.interface)
 
-    def specialise(self, subclass):
+    def copy_properties(self, symbol_in):
+        '''Replace all properties in this object with the properties from
+        symbol_in, apart from the name (which is immutable) and visibility.
+
+        :param symbol_in: the symbol from which the properties are copied.
+        :type symbol_in: :py:class:`psyclone.psyir.symbols.Symbol`
+
+        :raises TypeError: if the argument is not the expected type.
+
+        '''
+        if not isinstance(symbol_in, Symbol):
+            raise TypeError("Argument should be of type 'Symbol' but "
+                            "found '{0}'.".format(type(symbol_in).__name__))
+        self._interface = symbol_in.interface
+
+    def specialise(self, subclass, **kwargs):
         '''Specialise this symbol so that it becomes an instance of the class
         provided in the subclass argument. This allows this instance
         to become a subclass without any references to it becoming
@@ -282,19 +314,21 @@ class Symbol(object):
                 "but found '{2}'.".format(
                     self.name, type(self).__name__, subclass.__name__))
         self.__class__ = subclass
+        self._process_arguments(**kwargs)
 
+    # pylint: disable=inconsistent-return-statements
     def get_external_symbol(self):
         '''
-        Looks-up and returns the Symbol referred to by this Symbol's external
-        (global) interface.
+        Looks-up and returns the Symbol referred to by this Symbol's
+        Import Interface.
 
         :raises SymbolError: if the module pointed to by the symbol interface \
                              does not contain the symbol (or the symbol is \
                              not public).
-        :raises NotImplementedError: if the this symbol does not have an \
-                                     external (global) interface.
+        :raises NotImplementedError: if this symbol does not have an \
+                                     ImportInterface.
         '''
-        if not self.is_global:
+        if not self.is_import:
             raise NotImplementedError(
                 "Error trying to resolve symbol '{0}' properties, the lazy"
                 " evaluation of '{1}' interfaces is not supported."
@@ -323,7 +357,7 @@ class Symbol(object):
         class and type of the looked-up symbol are the same as this one,
         some specialisations of this method update the differing
         properties in place rather than create a new symbol.
-        If this symbol does not have a 'global' interface then there is no
+        If this symbol does not have an ImportInterface then there is no
         lookup needed and we just return this symbol.
 
         :returns: a symbol object with the class and type determined by \
@@ -331,7 +365,7 @@ class Symbol(object):
         :rtype: subclass of :py:class:`psyclone.psyir.symbols.Symbol`
 
         '''
-        if self.is_global:
+        if self.is_import:
             extern_symbol = self.get_external_symbol()
             # Create a new symbol object of the same class as the one
             # we've just looked up but with the interface and visibility
@@ -410,13 +444,13 @@ class Symbol(object):
         return isinstance(self._interface, LocalInterface)
 
     @property
-    def is_global(self):
+    def is_import(self):
         '''
-        :returns: whether the Symbol has a Global interface.
+        :returns: whether the Symbol has an ImportInterface.
         :rtype: bool
 
         '''
-        return isinstance(self._interface, GlobalInterface)
+        return isinstance(self._interface, ImportInterface)
 
     @property
     def is_argument(self):
@@ -476,3 +510,102 @@ class Symbol(object):
 
     def __str__(self):
         return self.name
+
+    @property
+    def is_array(self):
+        '''This is the basic implementation for the method that checks
+        if a symbol is declared to be an array. In this base class it
+        will just raise an exception, indicating that no information
+        is available. The function will be overwritten, e.g. in
+        DataSymbol.
+
+        :returns: True if this symbol is an array and False otherwise.
+        :rtype: bool
+
+        :raises ValueError: if this function is called for the base class \
+            since there is no information available.
+
+        '''
+        raise ValueError("No array information is available for the "
+                         "symbol '{0}'.".format(self.name))
+
+    def is_array_access(self, index_variable=None, access_info=None):
+        '''This method detects if a variable is used as an array or not.
+        If available, it will use the access information, i.e. how the
+        variable is used (e.g. if it has indices somewhere, like `a(i)%b`).
+        This can be incorrect in case of implicit loops (e.g. `a=b+1`,
+        where `a` and `b` are arrays) where the variable usage information
+        will not have information about indices. In this case this function
+        will fallback to querying the symbol itself.
+
+        This can cause significant slowdown if this symbol is imported
+        from a module, since then these modules need to be parsed.
+        # TODO #1213: Parsing modules is not yet supported.
+
+        If a `loop_variable` is specified, a variable access will only be
+        considered an array access if the specified variable is used in
+        at least one of the indices. For example:
+        >>> do i=1, n
+        >>>     a(j) = 2
+
+        the access to `a` is not considered an array if `loop_variable` is
+        set to `i`. If `loop_variable` is specified, `access_information`
+        must be specified.
+
+        :param str index_variable: optional loop variable that is used to \
+            to determine if an access is an array access using this variable.
+        :param access_info: variable access information, optional.
+        :type access_info: \
+            :py:class:`psyclone.core.access_info.SingleVariableAccessInfo`
+
+        :returns: if the variable is an array.
+        :rtype bool:
+
+        :raises InternalError: if a loop_variable is specified, but no \
+            access information is given.
+        :raises InternalError: if a loop_variable is specified, but no \
+            access information is given.
+
+        '''
+        # TODO #1270: this function might either be better off elsewhere,
+        # or even do not implement one function that uses both access
+        # information and symbol table - if required, the user can
+        # query both in two simple statements anyway.
+        if index_variable and not access_info:
+            raise InternalError("In Symbol.is_array_access: index variable "
+                                "'{0}' specified, but no access information "
+                                "given."
+                                .format(index_variable))
+
+        # TODO #1244: If as a result of 1244 we have more reliable
+        # information in the symbol table, the implementation here might
+        # be changed.
+
+        # Prioritise access information, since this can take the index
+        # variable into account, and avoids potentially parsing other
+        # modules to find type information.
+        if access_info:
+            # Access Info might not have information if a variable is used
+            # as array (e.g. in case of an array expression). In this case
+            # we still need to check the type information in the symbol table.
+            is_array = access_info.is_array(index_variable=index_variable)
+
+            # Access information might indicate that a variable is not an
+            # array if it is used in array expressions only. In order to
+            # verify, we need to check the symbol type information. But
+            # if an index variable is specified, an array used in an array
+            # expression without the index variable is not considered to
+            # be an array access, since it is independent of the loop
+            # variable. In this case also return the value from `is_array`.
+            if is_array or index_variable is not None:
+                return is_array
+
+        # Either we don't have access information, or the access information
+        # does not indicate an array. In the latter case we still need to
+        # test the symbol table, since the variable might be used in array
+        # expressions only. Note that we cannot check for index variable usage
+        # in this case. If there is no type information available (i.e. `self`
+        # is just a Symbol, not a DataSymbol), the `is_array` function will
+        # raise an exception.
+        # TODO #1213: check for wildcard imports
+        return self.is_array
